@@ -1,7 +1,6 @@
 /* libpq.c
 
-   Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009,
-     2010, 2011, 2012, 2013 Thien-Thi Nguyen
+   Copyright (C) 2003-2015 Thien-Thi Nguyen
    Portions Copyright (C) 1999, 2000 Ian Grant
 
    This file is part of Guile-PG.
@@ -17,8 +16,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with Guile-PG; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA  */
+   along with Guile-PG.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include "config.h"
 #include <stdlib.h>
@@ -27,18 +25,10 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#ifdef HAVE_POSTGRESQL_LIBPQ_FE_H
-#include <postgresql/libpq-fe.h>
-#else
 #include <libpq-fe.h>
-#endif
-#ifdef HAVE_POSTGRESQL_LIBPQ_LIBPQ_FS_H
-#include <postgresql/libpq/libpq-fs.h>
-#else
 #include <libpq/libpq-fs.h>
-#endif
-
 #include "gi.h"                         /* Guile interface */
+#include "snuggle/finangle.h"
 
 #ifdef __GNUC__
 #define UNUSED __attribute__ ((unused))
@@ -50,78 +40,9 @@
 
 #define TRUE_ENOUGH(svar)   (GIVENP (svar) && NOT_FALSEP (svar))
 
-
-/* Abstractions for Scheme objects to C string conversion.  */
+#define DECLARE_SMOB_NAME(cpre,string)                  \
+  static const char cpre ## _name[] = "PG-" string
 
-typedef struct {
-  char *s;
-  size_t len;
-} range_t;
-
-#define RS(svar)    c ## svar .s
-#define RLEN(svar)  c ## svar .len
-
-#if GI_LEVEL_NOT_YET_1_8
-
-#define FINANGLABLE_SCHEME_STRING_FROM_SYMBOL(sym)      \
-  scm_string_copy (scm_symbol_to_string (sym))
-
-/* Coerce a Scheme (sub)string that is to be used in contexts where the
-   extracted C string is expected to be read-only and zero-terminated.  We
-   check this condition precisely instead of simply coercing all (sub)strings,
-   to avoid waste for those substrings that may in fact (by lucky accident)
-   already satisfy the condition.  */
-#define ROZT_X(x)                                       \
-  if (SCM_ROCHARS (x) [SCM_ROLENGTH (x)])               \
-    x = BSTRING (SCM_ROCHARS (x), SCM_ROLENGTH (x))
-
-#define _FINANGLE(svar,p1)  do                  \
-    {                                           \
-      if (p1)                                   \
-        ROZT_X (svar);                          \
-      RS (svar) = SCM_ROCHARS (svar);           \
-      RLEN (svar) = SCM_ROLENGTH (svar);        \
-    }                                           \
-  while (0)
-
-#define UNFINANGLE(svar)
-
-#else  /* !GI_LEVEL_NOT_YET_1_8 */
-
-#define FINANGLABLE_SCHEME_STRING_FROM_SYMBOL  scm_symbol_to_string
-
-#define REND(svar)          RS (svar) [RLEN (svar)]
-#define NUL_AT_END_X(svar)  REND (svar) = '\0'
-
-static void
-_finangle (SCM svar, range_t *r, int p1)
-{
-  r->s = scm_to_locale_stringn (svar, &r->len);
-  if (r->s)
-    {
-      if (p1 && r->s[r->len])
-        {
-          r->s = realloc (r->s, 1 + r->len);
-          r->s[r->len] = '\0';
-        }
-    }
-  else
-    r->s = strdup ("");
-}
-
-#define _FINANGLE(svar,p1)  _finangle (svar, &c ## svar, p1)
-
-#define UNFINANGLE(svar)  free (RS (svar))
-
-#endif  /* !GI_LEVEL_NOT_YET_1_8 */
-
-/* Use ‘FINANGLE_RAW’ when the consumer of the C string takes full range
-   (start address plus length) info.  Otherwise, ‘FINANGLE’.  */
-
-#define FINANGLE_RAW(svar)  _FINANGLE (svar, 0)
-#define FINANGLE(svar)      _FINANGLE (svar, 1)
-
-
 /*
  * smob: connection
  */
@@ -152,7 +73,7 @@ xc_unbox (SCM obj)
 
 #define CONN_CONN(conn)  (xc_unbox (conn)->dbconn)
 
-static char xc_name[] = "PG-CONN";
+DECLARE_SMOB_NAME (xc, "CONN");
 
 static int
 xc_display (SCM exp, SCM port, UNUSED scm_print_state *pstate)
@@ -262,25 +183,39 @@ extract_mode_bits (SCM modes, int *appendp)
     }                                           \
   while (0)
 
-static char lob_name[] = "PG-LO-PORT";
+DECLARE_SMOB_NAME (lob, "LO-PORT");
+
+/* This actually appeared somewhere between Guile 1.6.0 and 1.6.8,
+   but the exact release number is too much bother to dig up at the
+   moment.  */
+#if !GI_LEVEL_1_8
+static SCM
+scm_new_port_table_entry (long tag)
+{
+  SCM z;
+  scm_port *pt;
+
+  NEWCELL_X (z);
+  SCM_SET_CELL_WORD_0 (z, tag);
+  pt = scm_add_to_port_table (z);
+  SCM_SETPTAB_ENTRY (z, pt);
+  return z;
+}
+#endif
 
 static SCM
 lob_mklobport (SCM conn, Oid oid, int alod, long modes, const char *FUNC_NAME)
 {
-  SCM port;
+  SCM port = scm_new_port_table_entry (lobp_tag);
   lob_stream *lobp;
-  scm_port *pt;
+  scm_port *pt = SCM_PTAB_ENTRY (port);;
 
   lobp = GCMALLOC (sizeof (lob_stream), lob_name);
 
-  NEWCELL_X (port);
-
   NOINTS ();
   lobp->conn = conn;
   lobp->oid = oid;
   lobp->alod = alod;
-  pt = scm_add_to_port_table (port);
-  SCM_SETPTAB_ENTRY (port, pt);
   SCM_SET_CELL_WORD_0 (port, lobp_tag | modes);
   SCM_SETSTREAM (port, (SCM) lobp);
 
@@ -475,10 +410,12 @@ from @code{pg-lo-creat} or @code{pg-lo-open}.  */)
 {
 #define FUNC_NAME s_pg_lo_get_oid
   ASSERT_PORT (1, port, LOBPORTP);
-  return NUM_INT (LOB_STREAM (port)->oid);
+  return NUM_ULONG (LOB_STREAM (port)->oid);
 #undef FUNC_NAME
 }
 
+static OFF_T lob_seek (SCM port, OFF_T offset, int whence);
+
 PRIMPROC
 (pg_lo_tell, "pg-lo-tell", 1, 0, 0,
  (SCM port),
@@ -494,7 +431,9 @@ explain what went wrong.  */)
 #define FUNC_NAME s_pg_lo_tell
   ASSERT_PORT (1, port, OPLOBPORTP);
 
-  return scm_ftell (port);
+  /* We used to use ‘scm_ftell’ here, but that is prone to segfault,
+     at least under Guile 2.0.9.  The price of progress...  */
+  return NUM_LONG (lob_seek (port, 0, SEEK_CUR));
 #undef FUNC_NAME
 }
 
@@ -591,13 +530,13 @@ lob_end_input (SCM port, int offset)
 #undef FUNC_NAME
 }
 
-static off_t
-lob_seek (SCM port, off_t offset, int whence)
+static OFF_T
+lob_seek (SCM port, OFF_T offset, int whence)
 {
 #define FUNC_NAME __func__
   lob_stream *lobp = LOB_STREAM (port);
   PGconn *conn = LOB_CONN (lobp);
-  off_t ret;
+  OFF_T ret;
 
   NOINTS ();
   ret = lo_lseek (conn, lobp->alod, offset, whence);
@@ -640,13 +579,15 @@ relative to the beginning of the object, or a number less than
 zero if an error occurred.  */)
 {
 #define FUNC_NAME s_pg_lo_seek
+  int cwhence;
+
   ASSERT_PORT (1, port, OPLOBPORTP);
-  ASSERT_EXACT (2, where);
-  ASSERT_EXACT (3, whence);
+  ASSERT_INTEGER (2, where);
+  VALIDATE_NNINT_COPY (3, whence, cwhence);
 
   lob_flush (port);
 
-  return NUM_INT (lob_seek (port, C_INT (where), C_INT (whence)));
+  return NUM_LONG (lob_seek (port, C_LONG (where), cwhence));
 #undef FUNC_NAME
 }
 
@@ -724,10 +665,20 @@ lob_input_waiting_p (UNUSED SCM port)
   return 1;
 }
 
+/* Define this here because ‘pg-lo-read’ is going away RSN; there's
+   no point in (even eventually) bubbling this up to Guile-BAUX.  */
+#if GI_LEVEL (1, 9)
+#define TAKE_STR  scm_take_locale_stringn
+#else
+#define TAKE_STR  scm_take_str
+#endif
+
 PRIMPROC
 (pg_lo_read, "pg-lo-read", 3, 0, 0,
  (SCM siz, SCM num, SCM port),
  doc: /***********
+@zonkable{2015-12-31,procedure}
+
 Read @var{num} objects each of length @var{siz} from @var{port}.
 Return a string containing the data read from the port or
 @code{#f} if an error occurred.  */)
@@ -736,8 +687,8 @@ Return a string containing the data read from the port or
   char *stage, *wp;
   int csiz, cnum, len, c;
 
-  ASSERT_EXACT_NON_NEGATIVE_COPY (1, siz, csiz);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (2, num, cnum);
+  VALIDATE_NNINT_COPY (1, siz, csiz);
+  VALIDATE_NNINT_COPY (2, num, cnum);
   ASSERT_PORT (3, port, OPINLOBPORTP);
 
   if (0 > (len = csiz * cnum)
@@ -747,7 +698,8 @@ Return a string containing the data read from the port or
   while (len-- && (EOF != (c = scm_getc (port))))
     *wp++ = c;
   *wp = '\0';
-  return scm_take_str (stage, wp - stage);
+  /* TODO: Cascade zonking to ‘TAKE_STR’ definition, too.  */
+  return TAKE_STR (stage, wp - stage);
 #undef FUNC_NAME
 }
 
@@ -783,10 +735,9 @@ static size_t
 lob_free (SCM port)
 {
   lob_stream *lobp = LOB_STREAM (port);
-  int ret;
 
   if (SCM_OPENP (port))
-    ret = lob_close (port);
+    lob_close (port);
   GCFREE (lobp, lob_name);
   return GCRV (lobp);
 }
@@ -818,7 +769,7 @@ the failure.  */)
   INTSOK ();
 
   return DEFAULT_FALSE (InvalidOid != ret,
-                        NUM_INT (ret));
+                        NUM_ULONG (ret));
 #undef FUNC_NAME
 }
 
@@ -909,7 +860,7 @@ res_box (PGresult *res)
     RETURN_FALSE ();
 }
 
-static char res_name[] = "PG-RESULT";
+DECLARE_SMOB_NAME (res, "RESULT");
 
 static int
 res_display (SCM exp, SCM port, UNUSED scm_print_state *pstate)
@@ -967,6 +918,18 @@ strip_newlines (char *str)
   return BSTRING (str, lc + 1 - str);
 }
 
+static char *
+copy_string (SCM orig)                  /* DWR: needs ‘free’!  */
+{
+  char *rv = NULL;
+  range_t corig;
+
+  FINANGLE (orig);
+  rv = strndup (RS (orig), RLEN (orig));
+  UNFINANGLE (orig);
+  return rv;
+}
+
 
 /*
  * common parameter handling
@@ -1003,7 +966,7 @@ prep_paramspecs (const char *FUNC_NAME, struct paramspecs *ps, SCM v)
   ps->len = len = VECTOR_LEN (v);
   for (i = 0; i < len; i++)
     {
-      elem = VREF (v, i);
+      elem = VECTOR_REF (v, i);
       if (! STRINGP (elem))
         ERROR ("bad parameter-vector element: ~S", elem);
     }
@@ -1012,14 +975,7 @@ prep_paramspecs (const char *FUNC_NAME, struct paramspecs *ps, SCM v)
   if (! ps->values)
     MEMORY_ERROR ();
   for (i = 0; i < len; i++)
-    {
-      range_t celem;
-
-      elem = VREF (v, i);
-      FINANGLE (elem);
-      ps->values[i] = strdup (RS (elem));
-      UNFINANGLE (elem);
-    }
+    ps->values[i] = copy_string (VECTOR_REF (v, i));
   ps->lengths = NULL;
   ps->formats = NULL;
 }
@@ -1027,19 +983,37 @@ prep_paramspecs (const char *FUNC_NAME, struct paramspecs *ps, SCM v)
 static void
 drop_paramspecs (struct paramspecs *ps)
 {
+  int i;
+
   free (ps->types);
+  for (i = 0; i < ps->len; i++)
+    free ((void *) ps->values[i]);
   free (ps->values);
   free (ps->lengths);
   free (ps->formats);
 }
 
+#define PREP_PARAMSPECS(svar)  do               \
+    {                                           \
+      prep_paramspecs (FUNC_NAME, &ps, svar);   \
+      NOINTS ();                                \
+    }                                           \
+  while (0)
+
+#define DROP_PARAMSPECS()  do                   \
+    {                                           \
+      INTSOK ();                                \
+      drop_paramspecs (&ps);                    \
+    }                                           \
+  while (0)
+
 
 /*
  * other abstractions
  */
 
 #define VALIDATE_FIELD_NUMBER_COPY(pos,num,res,cvar)                    \
-  VALIDATE_EXACT_0_UP_TO_N_COPY (pos, num, PQnfields (res), cvar)
+  VALIDATE_NNINT_RANGE_COPY (pos, num, PQnfields (res), cvar)
 
 
 /*
@@ -1173,7 +1147,7 @@ presenting this option in the first place.  */)
 
   for (head = opt = PQconndefaults (); opt && opt->keyword; opt++)
     rv = CONS
-      (PCHAIN (scm_c_make_keyword (opt->keyword),
+      (PCHAIN (KEYWORD (opt->keyword),
                PAIRM (envvar,   STRING (opt->envvar)),
                PAIRM (compiled, STRING (opt->compiled)),
                PAIRM (val,      STRING (opt->val)),
@@ -1447,7 +1421,7 @@ If there is an error, return @code{#f}.  */)
 
   olen = PQescapeStringConn (dbconn, answer, RS (string), RLEN (string), &errcode);
   rv = DEFAULT_FALSE (! errcode,
-                      STRING (answer));
+                      BSTRING (answer, olen));
   free (answer);
   UNFINANGLE (string);
   return rv;
@@ -1571,15 +1545,13 @@ parameterized string, and @var{parms} is a parameter-vector.  */)
 
   VALIDATE_PARAM_RELATED_ARGS (statement);
 
-  prep_paramspecs (FUNC_NAME, &ps, parms);
-  NOINTS ();
+  PREP_PARAMSPECS (parms);
   result = PQexecParams (dbconn, RS (statement), ps.len,
                          ps.types, ps.values, ps.lengths, ps.formats,
                          RESFMT_TEXT);
   UNFINANGLE (statement);
   z = res_box (result);
-  INTSOK ();
-  drop_paramspecs (&ps);
+  DROP_PARAMSPECS ();
   return z;
 #undef FUNC_NAME
 }
@@ -1603,15 +1575,13 @@ name specified in some prior SQL @code{PREPARE} statement.
 
   VALIDATE_PARAM_RELATED_ARGS (stname);
 
-  prep_paramspecs (FUNC_NAME, &ps, parms);
-  NOINTS ();
+  PREP_PARAMSPECS (parms);
   result = PQexecPrepared (dbconn, RS (stname), ps.len,
                            ps.values, ps.lengths, ps.formats,
                            RESFMT_TEXT);
   UNFINANGLE (stname);
   z = res_box (result);
-  INTSOK ();
-  drop_paramspecs (&ps);
+  DROP_PARAMSPECS ();
   return z;
 #undef FUNC_NAME
 }
@@ -1702,7 +1672,7 @@ A symbol.
         {
         case PG_DIAG_STATEMENT_POSITION:
         case PG_DIAG_SOURCE_LINE:
-          rv = EVAL_STRING (s);
+          rv = NUM_INT (atoi (s));
           break;
         case PG_DIAG_SOURCE_FUNCTION:
           rv = SYMBOL (s);
@@ -1710,6 +1680,8 @@ A symbol.
         default:
           rv = STRING (s);
         }
+
+      return rv;
     }
 
   RETURN_FALSE ();
@@ -1742,7 +1714,7 @@ PRIMPROC
  doc: /***********
 Return the most-recent error message that occurred on the
 connection @var{conn}, or an empty string.
-For backward compatability, if @var{conn} is actually
+For backward compatibility, if @var{conn} is actually
 a result object returned from calling @code{pg-exec},
 delegate the call to @code{pg-result-error-message}
 transparently (new code should call that procedure
@@ -2091,7 +2063,7 @@ return the integer @acronym{OID} of the inserted tuple, otherwise
   INTSOK ();
 
   return DEFAULT_FALSE (InvalidOid != oid_value,
-                        NUM_INT (oid_value));
+                        NUM_ULONG (oid_value));
 #undef FUNC_NAME
 }
 
@@ -2211,13 +2183,11 @@ not valid for the given @code{result}.  */)
 #define FUNC_NAME s_pg_ftype
   PGresult *res;
   int field;
-  int ftype;
 
   VALIDATE_RESULT_UNBOX (1, result, res);
   VALIDATE_FIELD_NUMBER_COPY (2, num, res, field);
-  ftype = PQftype (res, field);
 
-  return NUM_INT (ftype);
+  return NUM_ULONG (PQftype (res, field));
 #undef FUNC_NAME
 }
 
@@ -2263,8 +2233,8 @@ up to the caller to convert this to the required type.  */)
   SCM rv;
 
   VALIDATE_RESULT_UNBOX (1, result, res);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (2, stuple, ctuple);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (3, sfield, cfield);
+  VALIDATE_NNINT_COPY (2, stuple, ctuple);
+  VALIDATE_NNINT_COPY (3, sfield, cfield);
   CHECK_TUPLE_COORDS ();
 
   NOINTS ();
@@ -2291,8 +2261,8 @@ Return the size in bytes of the value of the attribute
   SCM ret;
 
   VALIDATE_RESULT_UNBOX (1, result, res);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (2, stuple, ctuple);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (3, sfield, cfield);
+  VALIDATE_NNINT_COPY (2, stuple, ctuple);
+  VALIDATE_NNINT_COPY (3, sfield, cfield);
   CHECK_TUPLE_COORDS ();
 
   NOINTS ();
@@ -2317,8 +2287,8 @@ Return @code{#t} if the value of the attribute @var{sfield}, tuple
   SCM rv;
 
   VALIDATE_RESULT_UNBOX (1, result, res);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (2, stuple, ctuple);
-  ASSERT_EXACT_NON_NEGATIVE_COPY (3, sfield, cfield);
+  VALIDATE_NNINT_COPY (2, stuple, ctuple);
+  VALIDATE_NNINT_COPY (3, sfield, cfield);
   CHECK_TUPLE_COORDS ();
 
   NOINTS ();
@@ -2461,7 +2431,7 @@ done; or @code{-2} to mean an error occurred.
       if (pwritep)
         WBPORT (port, newbuf, rv);
       if (swritep)
-        SETCAR (port, BSTRING (newbuf, rv));
+        scm_set_car_x (port, BSTRING (newbuf, rv));
     }
   INTSOK ();
   PQfreemem (newbuf);
@@ -2470,145 +2440,6 @@ done; or @code{-2} to mean an error occurred.
 #undef FUNC_NAME
 }
 
-PRIMPROC
-(pg_getline, "pg-getline", 1, 0, 0,
- (SCM conn),
- doc: /***********
-Read a line from @var{conn} on which a @code{COPY <table> TO
-STDOUT} has been issued.  Return a string from the connection.
-A returned string consisting of a backslash followed by a full
-stop signifies an end-of-copy marker.
-
-This procedure is obsolete and @strong{WILL BE REMOVED}
-by 2013-12-31.  */)
-{
-#define FUNC_NAME s_pg_getline
-  PGconn *dbconn;
-  char buf[BUF_LEN];
-  int ret = 1;
-  SCM box = PCHAIN (SCM_UNSPECIFIED);
-  SCM tp = box;
-
-  VALIDATE_CONNECTION_UNBOX_DBCONN (1, conn, dbconn);
-  while (ret != 0 && ret != EOF)
-    {
-      NOINTS ();
-      ret = PQgetline (dbconn, buf, BUF_LEN);
-      INTSOK ();
-      SETCDR (tp, CONS (STRING (buf), SCM_EOL));
-      tp = CDR (tp);
-    }
-  return scm_string_append (CDR (box));
-#undef FUNC_NAME
-}
-
-PRIMPROC
-(pg_getlineasync, "pg-getlineasync", 2, 1, 0,
- (SCM conn, SCM buf, SCM tickle),
- doc: /***********
-Read a line from @var{conn} on which a @code{COPY <table> TO
-STDOUT} has been issued, into @var{buf} (a string).
-Return @code{-1} to mean end-of-copy marker recognized, or a number
-(possibly zero) indicating how many bytes of data were read.
-The returned data may contain at most one newline (in the last
-byte position).
-Optional arg @var{tickle} non-@code{#f} means to do a
-``consume input'' operation prior to the read.
-
-This procedure is obsolete and @strong{WILL BE REMOVED}
-by 2013-12-31.  */)
-{
-#define FUNC_NAME s_pg_getlineasync
-  PGconn *dbconn;
-  int rv;
-
-  VALIDATE_CONNECTION_UNBOX_DBCONN (1, conn, dbconn);
-  ASSERT_STRING (2, buf);
-
-  if (TRUE_ENOUGH (tickle))
-    /* We don't care if there was an error consuming input; caller can use
-       ‘pg_error_message’ to find out afterwards, or simply avoid tickling in
-       the first place.  */
-    PQconsumeInput (dbconn);
-
-#if GI_LEVEL_NOT_YET_1_8
-  rv = PQgetlineAsync (dbconn, SCM_ROCHARS (buf), SCM_ROLENGTH (buf));
-#else  /* !GI_LEVEL_NOT_YET_1_8 */
-  /* Fill a temporary C buffer then copy into the Scheme string.
-     Totally gross!  */
-  {
-    size_t sz = scm_c_string_length (buf);
-    char *cbuf = malloc (sz);
-
-    if (! cbuf)
-      SYSTEM_ERROR ();
-    rv = PQgetlineAsync (dbconn, cbuf, sz);
-    while (sz--)
-      scm_c_string_set_x (buf, sz, CHARACTER (cbuf[sz]));
-    free (cbuf);
-  }
-#endif /* !GI_LEVEL_NOT_YET_1_8 */
-
-  return NUM_INT (rv);
-#undef FUNC_NAME
-}
-
-PRIMPROC
-(pg_putline, "pg-putline", 2, 0, 0,
- (SCM conn, SCM str),
- doc: /***********
-Write a line to the connection on which a @code{COPY <table>
-FROM STDIN} has been issued.  The lines written should include
-the final newline characters.  The last line should be a
-backslash, followed by a @samp{.} (full-stop).  After this, the
-@code{pg-endcopy} procedure should be called for this
-connection before any further @code{pg-exec} call is made.
-Return @code{#t} if successful.
-
-This procedure is obsolete and @strong{WILL BE REMOVED}
-by 2013-12-31.  */)
-{
-#define FUNC_NAME s_pg_putline
-  PGconn *dbconn;
-  int status;
-  range_t cstr;
-
-  VALIDATE_CONNECTION_UNBOX_DBCONN (1, conn, dbconn);
-  ASSERT_STRING (2, str);
-  NOINTS ();
-  FINANGLE_RAW (str);
-  status = PQputnbytes (dbconn, RS (str), RLEN (str));
-  UNFINANGLE (str);
-  INTSOK ();
-  return BOOLEAN (! status);
-#undef FUNC_NAME
-}
-
-PRIMPROC
-(pg_endcopy, "pg-endcopy", 1, 0, 0,
- (SCM conn),
- doc: /***********
-Resynchronize with the backend process on @var{conn}.  This procedure
-must be called after the last line of a table has been
-transferred using @code{pg-getline}, @code{pg-getlineasync}
-or @code{pg-putline}.  Return @code{#t} if successful.
-
-This procedure is obsolete and @strong{WILL BE REMOVED}
-by 2013-12-31.  */)
-{
-#define FUNC_NAME s_pg_endcopy
-  PGconn *dbconn;
-  int ret;
-
-  VALIDATE_CONNECTION_UNBOX_DBCONN (1, conn, dbconn);
-  NOINTS ();
-  ret = PQendcopy (dbconn);
-  INTSOK ();
-
-  return BOOLEAN (! ret);
-#undef FUNC_NAME
-}
-
 SIMPLE_KEYWORD (terse);
 SIMPLE_KEYWORD (default);
 SIMPLE_KEYWORD (verbose);
@@ -2735,7 +2566,7 @@ sepo_unbox (SCM obj)
   return SMOBDATA (obj);
 }
 
-static char sepo_name[] = "PG-PRINT-OPTION";
+DECLARE_SMOB_NAME (sepo, "PRINT-OPTION");
 
 static size_t
 sepo_free (SCM obj)
@@ -2813,15 +2644,7 @@ option_as_string (SCM alist, SCM key, const char *def)
   SCM maybe = scm_assq_ref (alist, key);
 
   if (NOT_FALSEP (maybe))
-    {
-      range_t cmaybe;
-      char *rv;
-
-      FINANGLE (maybe);
-      rv = strdup (RS (maybe));
-      UNFINANGLE (maybe);
-      return rv;
-    }
+    return copy_string (maybe);
 
   if (def)
     return strdup (def);
@@ -2938,12 +2761,7 @@ To specify a disabled flag, use @dfn{no-FLAG}, e.g.,
       po->fieldName[count] = NULL;
       for (i = 0; i < count; i++)
         {
-          SCM name = CAR (substnames);
-          range_t cname;
-
-          FINANGLE (name);
-          po->fieldName[i] = strdup (RS (name));
-          UNFINANGLE (name);
+          po->fieldName[i] = copy_string (CAR (substnames));
           substnames = CDR (substnames);
         }
     }
@@ -3157,15 +2975,15 @@ is out of range.  If @var{start} is exactly the length of
       if (PROB (cenc))
         ERROR ("No such encoding: ~A", encoding);
       cell = CONS (encoding, NUM_INT (cenc));
-      SETCDR (encoding_alist, CONS (cell, CDR (encoding_alist)));
+      scm_set_cdr_x (encoding_alist, CONS (cell, CDR (encoding_alist)));
     }
 
   ASSERT_STRING (2, string);
   FINANGLE (string);
   if (GIVENP (start))
     {
-      ASSERT_EXACT (3, start);
-      cstart = C_INT (start);
+      ASSERT_INTEGER (3, start);
+      cstart = C_ULONG (start);
       if (RLEN (string) < cstart)
         ERROR ("String start index out of range: ~A", start);
     }
@@ -3188,11 +3006,9 @@ Return the current client encoding for @var{conn} as a string.  */)
 {
 #define FUNC_NAME s_pg_client_encoding
   PGconn *dbconn;
-  SCM enc;
 
   VALIDATE_CONNECTION_UNBOX_DBCONN (1, conn, dbconn);
-  enc = STRING (pg_encoding_to_char (PQclientEncoding (dbconn)));
-  return enc;
+  return STRING (pg_encoding_to_char (PQclientEncoding (dbconn)));
 #undef FUNC_NAME
 }
 
@@ -3300,14 +3116,12 @@ parameterized string, and @var{parms} is a parameter-vector.  */)
 
   VALIDATE_PARAM_RELATED_ARGS (query);
 
-  prep_paramspecs (FUNC_NAME, &ps, parms);
-  NOINTS ();
+  PREP_PARAMSPECS (parms);
   result = PQsendQueryParams (dbconn, RS (query), ps.len,
                               ps.types, ps.values, ps.lengths, ps.formats,
                               RESFMT_TEXT);
   UNFINANGLE (query);
-  INTSOK ();
-  drop_paramspecs (&ps);
+  DROP_PARAMSPECS ();
   return BOOLEAN (result);
 #undef FUNC_NAME
 }
@@ -3327,14 +3141,12 @@ Also, return @code{#t} if successful.  */)
 
   VALIDATE_PARAM_RELATED_ARGS (stname);
 
-  prep_paramspecs (FUNC_NAME, &ps, parms);
-  NOINTS ();
+  PREP_PARAMSPECS (parms);
   result = PQsendQueryPrepared (dbconn, RS (stname), ps.len,
                                 ps.values, ps.lengths, ps.formats,
                                 RESFMT_TEXT);
   UNFINANGLE (stname);
-  INTSOK ();
-  drop_paramspecs (&ps);
+  DROP_PARAMSPECS ();
   return BOOLEAN (result);
 #undef FUNC_NAME
 }
@@ -3502,16 +3314,8 @@ init_module (void)
 #undef KLIST
 }
 
-#ifdef USE_CMOD
 MOD_INIT_LINK_THUNK ("database postgres"
                      ,database_postgres
                      ,init_module)
-#else
-void
-guile_pg_init_database_postgres_module (void)
-{
-  init_module ();
-}
-#endif
 
 /* libpq.c ends here */
