@@ -1,7 +1,6 @@
 ;;; postgres-meta.scm --- Methods for understanding PostgreSQL data structures
 
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006,
-;;   2007, 2008, 2009, 2011 Thien-Thi Nguyen
+;; Copyright (C) 2002-2009, 2011, 2014, 2015 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -16,22 +15,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with Guile-PG; if not, write to the Free Software Foundation,
-;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-;;; Commentary:
-
-;; This module exports the procs:
-;;   (defs-from-psql PSQL DB-NAME TABLE-NAME) => defs
-;;   (check-type/elaborate TYPE) => #f, or KNOWN-TOBJ, or NEW-TYPE
-;;   (strictly-check-types/elaborate! TABLE-NAME TYPES) => #t, or ERROR
-;;   (infer-defs CONN TABLE-NAME) => defs
-;;   (describe-table! DB-NAME TABLE-NAME)
-;;
-;; DB-NAME and TABLE-NAME are strings.  DEF is a single column def.  DEFS is a
-;; list of column defs.  TYPE and NEW-TYPE are symbols.  KNOWN-TOBJ is the
-;; type object already registered.  CONN is a the result of ‘pg-connectdb’.
-;; ERROR means an error is thrown.  PSQL is either a string, a thunk, or #t.
+;; along with Guile-PG.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Code:
 
@@ -50,7 +34,7 @@
                 #:select (sql-command<-trees
                           parse+make-SELECT-tree))
   #:use-module ((database postgres-types)
-                #:select (dbcoltype-lookup
+                #:select (type-registered?
                           define-db-col-type
                           define-db-col-type-array-variant))
   #:use-module ((database postgres-resx)
@@ -644,7 +628,7 @@
           (loop (next) (cons def acc))))))
 
 ;; Check @var{type}, a symbol.  If it not an array variant, return
-;; non-@code{#f} only if its type converters are already registered with
+;; non-@code{#f} only if its converters are already registered with
 ;; Guile-PG.  If @var{type} is an array variant, check the base (non-array)
 ;; type first, and if needed, ensure that the array variant type is
 ;; registered.  Return non-@code{#f} if successful.
@@ -653,10 +637,10 @@
   (let* ((s (symbol->string type))
          (n (string-index s #\[))
          (base (string->symbol (if n (substring s 0 n) s))))
-    (and (dbcoltype-lookup base)
+    (and (type-registered? base)
          (or (not n)
              (let ((array-variant type))
-               (or (dbcoltype-lookup array-variant)
+               (or (type-registered? array-variant)
                    (begin
                      (define-db-col-type-array-variant array-variant base)
                      array-variant)))))))
@@ -702,7 +686,7 @@
       (relhaspkey     boolean ,nn)
       (relhasrules    boolean ,nn)
       (relhassubclass boolean ,nn)
-      (relacl         aclitem[]))))
+      (relacl         *aclitem))))
 
 (define (make-M:pg-class db-name)
   (pgtable-manager db-name "pg_class" *class-defs*))
@@ -743,7 +727,7 @@
 ;; connection @var{conn} and @var{table-name}.  The column names are exact.
 ;; The column types are incorrect for array types, which are described as
 ;; @code{_FOO}; there is currently no way to infer whether this means
-;; @code{FOO[]} or @code{FOO[][]}, etc, without looking at the table's data.
+;; @code{*FOO} or @code{**FOO}, etc, without looking at the table's data.
 ;; No type options are checked at this time.
 ;;
 (define (infer-defs conn table-name)
