@@ -1,7 +1,6 @@
 ;;; postgres-qcons.scm --- construct SELECT queries
 
-;; Copyright (C) 2005, 2006, 2007, 2008, 2009,
-;;   2010, 2011, 2012 Thien-Thi Nguyen
+;; Copyright (C) 2005-2012, 2014 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -16,29 +15,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with Guile-PG; if not, write to the Free Software Foundation,
-;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-;;; Commentary:
-
-;; This module exports the procedures:
-;;   (qcons-declare! category x . extra)
-;;   (sql-pre string) => string
-;;   (sql-pre? string) => boolean
-;;   (sql-unpre string) => string
-;;   (sql-quote string) => string
-;;   (make-comma-separated-tree proc ls [parens? [more-ls...]]) => tree
-;;   (make-WHERE-tree condition) => tree
-;;   (make-GROUP-BY-tree expressions) => tree
-;;   (make-HAVING-tree conditions) => tree
-;;   (make-ORDER-BY-tree orderings) => tree
-;;   (make-SELECT/COLS-tree cols) => tree
-;;   (make-FROM-tree froms) => tree
-;;   (make-SELECT/FROM/COLS-tree froms cols) => tree
-;;   (parse+make-SELECT/tail-tree plist) => tree
-;;   (parse+make-SELECT-tree composition cols/subs [tail...]) => tree
-;;   (sql<-trees . trees) => string
-;;   (sql-command<-trees . trees) => string
+;; along with Guile-PG.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Code:
 
@@ -63,6 +40,7 @@
             parse+make-SELECT-tree
             sql<-trees
             sql-command<-trees)
+  #:use-module ((database postgres-types) #:select (type-sql-name))
   #:use-module ((srfi srfi-13) #:select (string-index
                                          string-prefix?
                                          string-concatenate-reverse
@@ -447,6 +425,11 @@
                       (xsub (1+ dot) ra)
                       (from ra)))))))
 
+(define (pretty-type-name type)
+  (if (string? type)
+      type
+      (type-sql-name type)))
+
 (define (maybe-dq sym)
   ;; Hmmm, why not use ‘idquote’ also for this?
   (if (eq? '* sym)
@@ -526,7 +509,10 @@
                     (#f ,(caddr rest))))
              #:END))
       ((::)
-       (list #:CAST (paren (as (expr (cadr rest)) (car rest)))))
+       (list #:CAST (paren (as (expr (cadr rest))
+                               ;; Don't bother w/ ‘sql-pre’ here.
+                               (pretty-type-name
+                                (car rest))))))
       ;; special constructs
       ((in/set)
        (list (expr (car rest)) #:IN
