@@ -1,7 +1,6 @@
 ;;; postgres-table.scm --- abstract manipulation of a single PostgreSQL table
 
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006,
-;;   2007, 2008, 2009, 2011, 2012 Thien-Thi Nguyen
+;; Copyright (C) 2002-2009, 2011, 2012, 2014, 2015 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -16,16 +15,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with Guile-PG; if not, write to the Free Software Foundation,
-;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-;;; Commentary:
-
-;; This module exports these procedures:
-;;   (sql-pre string)
-;;   (pgtable-manager db-spec table-name defs)
-;;   (pgtable-worker db-spec table-name defs)
-;;   (compile-outspec spec defs)
+;; along with Guile-PG.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Code:
 
@@ -33,10 +23,9 @@
   #:export (pgtable-manager
             pgtable-worker
             compile-outspec)
-  #:use-module ((ice-9 common-list)
-                #:select (find-if
-                          pick-mappings
-                          remove-if))
+  #:use-module ((srfi srfi-1)
+                #:select (find
+                          filter-map))
   #:use-module ((database postgres)
                 #:select (pg-connection?
                           pg-connectdb
@@ -44,17 +33,18 @@
                           pg-exec pg-ntuples pg-nfields
                           pg-fname pg-getvalue))
   #:use-module ((database postgres-types)
-                #:select (dbcoltype-lookup
-                          dbcoltype:stringifier
-                          dbcoltype:default
-                          dbcoltype:objectifier))
+                #:select (type-registered?
+                          type-stringifier
+                          type-objectifier
+                          type-default
+                          type-sql-name))
   #:use-module ((database postgres-col-defs)
                 #:select (column-name
                           type-name
                           type-options
                           validate-def
                           objectifiers)
-                #:renamer (symbol-prefix-proc 'def:))
+                #:prefix def:)
   #:use-module ((database postgres-qcons)
                 #:select (sql-pre
                           sql-pre?
@@ -115,11 +105,34 @@
   (let ((objectifiers '()))
 
     (define (push! type)
+      (or (type-registered? type)
+          (bad-select-part type))
       (set! objectifiers
-            (cons (dbcoltype:objectifier (or (dbcoltype-lookup type)
-                                             (bad-select-part type)))
+            (cons (type-objectifier type)
                   objectifiers)))
 
+    (define (three-part type title expr)
+      (and (string? expr) (bad-select-part expr))
+      (push! (cond ((symbol? type)
+                    type)
+                   ((eq? #f type)
+                    'text)
+                   ((and (eq? #t type)
+                         (or (assq expr defs)
+                             (bad-select-part expr)))
+                    => def:type-name)
+                   ((and (pair? type)
+                         (eq? #t (car type))
+                         (symbol? (cdr type))
+                         (or (assq (cdr type) defs)
+                             (bad-select-part (cdr type))))
+                    => def:type-name)
+                   (else
+                    (bad-select-part type))))
+      (if title
+          (cons title expr)
+          expr))
+
     (define (munge x)
       (cond ((symbol? x)
              (or (and=> (assq x defs)
@@ -128,26 +141,7 @@
                  (bad-select-part x))
              x)
             ((and (list? x) (= 3 (length x)))
-             (apply-to-args
-              x (lambda (type title expr)
-                  (and (string? expr) (bad-select-part expr))
-                  (push! (cond ((symbol? type)
-                                type)
-                               ((eq? #f type)
-                                'text)
-                               ((and (eq? #t type)
-                                     (or (assq expr defs)
-                                         (bad-select-part expr)))
-                                => def:type-name)
-                               ((and (pair? type)
-                                     (eq? #t (car type))
-                                     (symbol? (cdr type))
-                                     (or (assq (cdr type) defs)
-                                         (bad-select-part (cdr type))))
-                                => def:type-name)
-                               (else
-                                (bad-select-part type))))
-                  (if title (cons title expr) expr))))
+             (apply three-part x))
             (else
              (bad-select-part x))))
 
@@ -265,7 +259,7 @@
   (or (and (pair? defs) (not (null? defs)))
       (error "malformed defs:" defs))
   (for-each (lambda (def)
-              (def:validate-def def dbcoltype-lookup))
+              (def:validate-def def type-registered?))
             defs)
   (let* ((conn (cond ((pg-connection? db-spec)
                       db-spec)
@@ -296,9 +290,9 @@
 
     (define (col-defs defs cols)
       (map (lambda (col)
-             (or (find-if (lambda (def)
-                            (eq? (def:column-name def) col))
-                          defs)
+             (or (find (lambda (def)
+                         (eq? (def:column-name def) col))
+                       defs)
                  (error "invalid field name:" col)))
            cols))
 
@@ -321,9 +315,8 @@
     (define (->db-insert-string db-col-type x)
       (or (and (sql-pre? x) x)
           (and (keyword? x) x)
-          (let* ((def (dbcoltype-lookup db-col-type))
-                 (s (or (false-if-exception ((dbcoltype:stringifier def) x))
-                        (dbcoltype:default def))))
+          (let ((s (or (false-if-exception ((type-stringifier db-col-type) x))
+                       (type-default db-col-type))))
             (or (string? s) (error "not a string:" s))
             (sql-pre (sql-quote s)))))
 
@@ -349,7 +342,7 @@
              ;; type.  Apparently, this was not handled automatically in
              ;; old PostgreSQL versions.  See PostgreSQL User Guide: The
              ;; Serial Types.
-             ,@(pick-mappings
+             ,@(filter-map
                 (lambda (def)
                   (and (eq? 'serial (def:type-name def))
                        `(#:SEQUENCE ,(fmt "~A_~A_seq"
@@ -361,7 +354,7 @@
       (xt #:CREATE #:TABLE dq-table-name
           (cseptree (lambda (def)
                       (list (symbol->qstring (def:column-name def))
-                            (symbol->string (def:type-name def))
+                            (type-sql-name (def:type-name def))
                             (def:type-options def)))
                     defs #t)))
 
