@@ -1,7 +1,6 @@
 ;;; postgres-types.scm --- convert PostgreSQL <-> Scheme objects
 
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006,
-;;   2007, 2008, 2009, 2011, 2012 Thien-Thi Nguyen
+;; Copyright (C) 2002-2009, 2011, 2012, 2014, 2015 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -16,36 +15,7 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with Guile-PG; if not, write to the Free Software Foundation,
-;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-
-;;; Commentary:
-
-;; This module provides the procs:
-;;  (oid-type-name-cache CONN . FRESH?) => ((OID . TYPE-NAME) ...)
-;;  (dbcoltypes) => list of names
-;;  (dbcoltype-lookup NAME) => tobj
-;;  (dbcoltype:stringifier TOBJ) => proc
-;;  (dbcoltype:objectifier TOBJ) => proc
-;;  (dbcoltype:default) => string
-;;  (define-db-col-type NAME DEFAULT STRINGIFY OBJECTIFY)
-;;  (define-db-col-type-array-variant COMPOSED SIMPLE [STRINGIFY [OBJECTIFY]])
-;;
-;; NAME, COMPOSED and SIMPLE are symbols naming a type.  COMPOSED is
-;; conventionally formed by appending square-bracket paris to SIMPLE.  For
-;; example, we can declare a two-dimensional array type of text elements:
-;;
-;;  (define-db-col-type-array-variant 'text[][] 'text ...)
-;;
-;; STRINGIFY is a procedure that takes a Scheme object and returns a string,
-;; suitable for sending to PostgreSQL.  OBJECTIFY does the opposite: it takes
-;; a string from PostgreSQL and returns a Scheme object.
-;;
-;; DEFAULT is a string.  TOBJ is a "type object" which should be considered
-;; opaque (and thus subject to change w/o warning).  Use the dbcoltype:foo
-;; procs to access the components.
-;;
-;; TODO: Look into "user-defined" types from PostgreSQL point of view.
+;; along with Guile-PG.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Code:
 
@@ -56,10 +26,19 @@
             dbcoltype:stringifier
             dbcoltype:default
             dbcoltype:objectifier
+            type-registered?
+            type-stringifier
+            type-default
+            type-objectifier
+            type-sql-name
             define-db-col-type
+            register-array-variant
             define-db-col-type-array-variant)
   #:autoload (database postgres) (pg-exec))
 
+(define (fs s . args)
+  (apply simple-format #f s args))
+
 (define o/t (make-object-property))     ; oid/type-name info
 
 ;; Query connection @var{conn} for oid/type-name info, caching results.
@@ -79,32 +58,99 @@
                        (acons (string->number (pg-getvalue res n 0))
                               (pg-getvalue res n 1)
                               acc)))))))
-  (cond ((and (not (null? opt)) (car opt)) (set! (o/t conn) (fresh)))
+  (define (fresh!)
+    (let ((alist (fresh)))
+      (set! (o/t conn) alist)
+      alist))
+  (cond ((and (not (null? opt)) (car opt)) (fresh!))
         ((o/t conn))
-        (else (set! (o/t conn) (fresh)))))
+        (else (fresh!))))
 
 ;; column types / definition
 
-(define *db-col-types* '())     ; (NAME . #(STRINGIFIER DEFAULT OBJECTIFIER))
+(define ALL
+  ;; key: NAME (symbol)
+  ;; val: #(STRINGIFIER DEFAULT OBJECTIFIER)
+  (make-hash-table))
 
-;; Return names of all registered type converters.
+(define ARRAY-VARIANT-INFO
+  ;; key: NAME (symbol)
+  ;; val: #(SQL-NAME RANK SIMPLE CANONICAL-NAME)
+  (make-hash-table))
+
+(define (av-lookup symbol)
+  (hashq-ref ARRAY-VARIANT-INFO symbol))
+
+(define (make-array-variant-info sql-name rank simple canonical-name)
+  (vector sql-name rank simple canonical-name))
+
+(define (av-sql-name  v) (vector-ref v 0))
+(define (av-rank      v) (vector-ref v 1))
+(define (av-simple    v) (vector-ref v 2))
+(define (av-canonical v) (vector-ref v 3))
+
+;; @zonkable{2015-12-31,procedure}
+;;
+;; Return names of all registered converters.
 ;;
 (define (dbcoltypes)
-  (map car *db-col-types*))
+  (hash-fold (lambda (key val ls)
+               (cons key ls))
+             '() ALL))
 
-;; Return a type-converter object given its @var{type-name}, a symbol.
-;; Return @code{#f} if no such t-c object by that name exists.
+;; @zonkable{2015-12-31,procedure}
+;;
+;; Return a converter object given its @var{type-name}, a symbol.
+;; Return @code{#f} if no such object by that name exists.
 ;;
 (define (dbcoltype-lookup type-name)
-  (assq-ref *db-col-types* type-name))
+  (hashq-ref ALL (cond ((av-lookup type-name) => av-canonical)
+                       (else                     type-name))))
 
-;; Extract stringifier from the type-converter object @var{tc}.
+;; @zonkable{2015-12-31,procedure}
+;;
+;; Extract stringifier from the converter object @var{tc}.
+;;
 (define (dbcoltype:stringifier tc) (vector-ref tc 0))
-;; Extract default string from the type-converter object @var{tc}.
+
+;; @zonkable{2015-12-31,procedure}
+;;
+;; Extract default string from the converter object @var{tc}.
+;;
 (define (dbcoltype:default tc) (vector-ref tc 1))
-;; Extract objectifier from the type-converter object @var{tc}.
+
+;; @zonkable{2015-12-31,procedure}
+;;
+;; Extract objectifier from the converter object @var{tc}.
+;;
 (define (dbcoltype:objectifier tc) (vector-ref tc 2))
 
+;; Return @code{#t} if @var{type} (a symbol) has registered converters.
+;;
+(define (type-registered? type)
+  (->bool (hashq-get-handle ALL type)))
+
+;; Return the stringifier for @var{type} (a symbol).
+;;
+(define (type-stringifier type)
+  (dbcoltype:stringifier (dbcoltype-lookup type)))
+
+;; Return the default for @var{type} (a symbol).
+;;
+(define (type-default type)
+  (dbcoltype:default (dbcoltype-lookup type)))
+
+;; Return the objectifier for @var{type} (a symbol).
+;;
+(define (type-objectifier type)
+  (dbcoltype:objectifier (dbcoltype-lookup type)))
+
+;; Return the SQL name (a string) of @var{type} (a symbol).
+;;
+(define (type-sql-name type)
+  (cond ((av-lookup type) => av-sql-name)
+        (else                (symbol->string type))))
+
 (define (read-pgarray-1 objectifier port)
   ;; ugh, i hate parsing...  the right thing to do would be find out if
   ;; postgres supports (easily!)  parameterizable array output formatting.
@@ -140,6 +186,24 @@
                             (read-char port)
                             (read-pgarray-1 objectifier port))))
 
+(define (dimension->string stringifier x)
+  (letrec ((dive (lambda (ls)
+                   (list "{" (dimension->string stringifier (car ls))
+                         (map (lambda (y)
+                                (list "," (dimension->string stringifier y)))
+                              (cdr ls))
+                         "}")))
+           (walk (lambda (x)
+                   (cond ((string? x) (display x))
+                         ((list? x) (for-each walk x))
+                         (else (error "bad type:" x)))))
+           (flatten (lambda (tree)
+                      (with-output-to-string
+                        (lambda () (walk tree))))))
+    (cond ((list? x)   (flatten (dive x)))
+          ((vector? x) (flatten (dive (vector->list x))))
+          (else        (stringifier x)))))
+
 (define (dimension->string-proc stringifier)
   (lambda (ls/vec) (dimension->string stringifier ls/vec)))
 
@@ -160,58 +224,102 @@
 ;; See also @code{dbcoltype-lookup}.
 ;;
 (define (define-db-col-type name default stringifier objectifier)
-  (set! *db-col-types*
-        (assq-set! *db-col-types* name
-                   (vector stringifier default objectifier))))
+  (hashq-set! ALL name
+              (vector stringifier default objectifier)))
 
-(define (dimension->string stringifier x)
-  (letrec ((dive (lambda (ls)
-                   (list "{" (dimension->string stringifier (car ls))
-                         (map (lambda (y)
-                                (list "," (dimension->string stringifier y)))
-                              (cdr ls))
-                         "}")))
-           (walk (lambda (x)
-                   (cond ((string? x) (display x))
-                         ((list? x) (for-each walk x))
-                         (else (error "bad type:" x)))))
-           (flatten (lambda (tree)
-                      (with-output-to-string
-                        (lambda () (walk tree))))))
-    (cond ((list? x)   (flatten (dive x)))
-          ((vector? x) (flatten (dive (vector->list x))))
-          (else        (stringifier x)))))
+;; Register an array type of @var{rank} dimensions based on @var{simple}.
+;; @var{rank} is a (typically small) positive integer.
+;; @var{simple} is a type name already
+;; registered using @code{define-db-col-type}.
+;;
+;; By default, the associated stringifier and objectifier are
+;; those of @var{simple}.  If @var{stringifier} and @var{objectifier}
+;; are specified and non-@code{#f}, Guile-PG uses them instead.
+;;
+;; The default value of all array types is @samp{@{@}} and cannot be changed.
+;;
+;; Return the name (a symbol) of the array type.  This is basically
+;; @var{rank} asterisks followed immediately by @var{simple}.  For example,
+;; if @var{rank} is 2 and and @var{simple} is @code{int4}, the name would
+;; be @code{**int4}.
+;;
+;;-args: (- 2 0 stringifier objectifier)
+;;
+(define (register-array-variant rank simple . procs)
 
-;; Register type @var{composed}, an array variant of @var{simple}, with
-;; optional @var{procs}.  @var{simple} should be a type name already
-;; registered using @code{define-db-col-type}.  @var{composed} is
-;; conventionally formed by appending @var{simple} with one or more pairs of
+  (define (array-variant-name)
+    (string-append (make-string rank #\*)
+                   (symbol->string simple)))
+
+  (define (sql-name)
+    (apply string-append
+           (symbol->string simple)
+           (make-list rank "[]")))
+
+  (or (type-registered? simple)
+      (error "unregistered type:" simple))
+  (let* ((stringifier (and (not (null? procs))
+                           (car procs)))
+         (objectifier (and (not (null? procs))
+                           (not (null? (cdr procs)))
+                           (cadr procs)))
+         (name (string->symbol (array-variant-name))))
+
+    (hashq-set! ARRAY-VARIANT-INFO name
+                (make-array-variant-info
+                 (sql-name) rank simple name))
+
+    (define-db-col-type name "{}"
+      (dimension->string-proc (or stringifier (type-stringifier simple)))
+      (read-array-string-proc (or objectifier (type-objectifier simple))))
+
+    ;; rv
+    name))
+
+;; @zonkable{2015-12-31,procedure}
+;;
+;; Register type @var{composed} (string or symbol)
+;; as an array variant of @var{simple}.
+;; @var{simple} should be a type name already
+;; registered using @code{define-db-col-type}.  @var{composed} @strong{must}
+;; be formed by appending @var{simple} with one or more pairs of
 ;; @samp{[]} (square braces), with the number of pairs indicating the array
 ;; dimensionality.  For example, if @var{simple} is @code{text}, a
 ;; two-dimensional text array would be named @code{text[][]}.
 ;;
-;; Optional arg @var{procs} is a list specifying alternative stringifier and
-;; objectifier procedures (in that order).  If unspecified, @var{simple} is
-;; looked up and its stringifier and objectifier are used.  See
-;; @code{dbcoltype-lookup}.
+;; By default, the stringifier and objectifier for @var{composed} are
+;; those of @var{simple}.  If @var{stringifier} and @var{objectifier}
+;; are specified and non-@code{#f}, Guile-PG uses them instead.
 ;;
 ;; The default value of all array types is @samp{@{@}} and cannot be changed.
 ;;
+;;-args: (- 2 0 stringifier objectifier)
+;;
 (define (define-db-col-type-array-variant composed simple . procs)
-  (let* ((lookup (dbcoltype-lookup simple))
-         (stringifier (or (and (not (null? procs))
-                               (car procs))
-                          (dbcoltype:stringifier lookup)))
-         (objectifier (or (and (not (null? procs))
-                               (not (null? (cdr procs)))
-                               (cadr procs))
-                          (dbcoltype:objectifier lookup))))
-    (define-db-col-type composed "{}"
-      (dimension->string-proc stringifier)
-      (read-array-string-proc objectifier))))
+  (and (string? composed)
+       (set! composed (string->symbol composed)))
+  (let* ((sql-name (symbol->string composed))
+         (neck (or (string-index sql-name #\[)
+                   (error "no ‘[]’ in composed:" composed)))
+         (rank (ash (- (string-length sql-name)
+                       neck)
+                    -1)))
+    ;; sanity checks
+    (or (eq? (string->symbol (substring sql-name 0 neck))
+             simple)
+        (error (fs "composed ‘~A’ not based on simple ‘~A’"
+                   composed simple)))
+    (or (positive? rank)
+        (error "malformed:" composed))
+    ;; do it!
+    (let ((name (apply register-array-variant rank simple procs)))
+      (define (add-ref ht)
+        (hashq-set! ht composed (hashq-ref ht name)))
+      (add-ref ALL)
+      (add-ref ARRAY-VARIANT-INFO))))
 
 ;;;---------------------------------------------------------------------------
-;;; load-time actions: set up *db-col-types*
+;;; load-time actions: set up built-ins
 
 ;; support
 
@@ -424,9 +532,9 @@
 ;; -- network address types
 
 (define (n+m-stringifier n+m)           ; n+m is #(NUMBER MASKCOUNT)
-  (simple-format #f "~A/~A"
-                 (inet-ntoa (vector-ref n+m 0))
-                 (vector-ref n+m 1)))
+  (fs "~A/~A"
+      (inet-ntoa (vector-ref n+m 0))
+      (vector-ref n+m 1)))
 
 (define (n+m-objectifier s)
   (let ((cut (string-index s #\/)))
@@ -448,7 +556,7 @@
   n+m-objectifier)
 
 (define (host-stringifier n)
-  (simple-format #f "~A/32" (inet-ntoa n)))
+  (fs "~A/32" (inet-ntoa n)))
 
 (define (host-objectifier s)
   (vector-ref (n+m-objectifier s) 0))
@@ -461,7 +569,7 @@
   (lambda (n)
     (let loop ((bpos 0) (acc '()) (n n))
       (if (= bpos 48)
-          (apply simple-format #f "~A:~A:~A:~A:~A:~A"
+          (apply fs "~A:~A:~A:~A:~A:~A"
                  (map (lambda (x) (number->string x 16)) acc))
           (loop (+ bpos 8)
                 (cons (logand #xff n) acc)
@@ -494,9 +602,9 @@
 
 ;; array variants
 
-(define-db-col-type-array-variant 'text[]   'text double-quote identity)
-(define-db-col-type-array-variant 'text[][] 'text double-quote identity)
-(define-db-col-type-array-variant 'int4[]   'int4)
-(define-db-col-type-array-variant 'aclitem[] 'aclitem double-quote identity)
+(define-db-col-type-array-variant "text[]"    'text double-quote identity)
+(define-db-col-type-array-variant "text[][]"  'text double-quote identity)
+(define-db-col-type-array-variant "int4[]"    'int4)
+(define-db-col-type-array-variant "aclitem[]" 'aclitem double-quote identity)
 
 ;;; postgres-types.scm ends here
