@@ -1,7 +1,6 @@
 ;;; types-table.scm                             -*- coding: utf-8 -*-
 
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006,
-;;   2007, 2008, 2009, 2011, 2012 Thien-Thi Nguyen
+;; Copyright (C) 2002-2009, 2011, 2012, 2014, 2015 Thien-Thi Nguyen
 ;;
 ;; This file is part of Guile-PG.
 ;;
@@ -16,20 +15,15 @@
 ;; GNU General Public License for more details.
 ;;
 ;; You should have received a copy of the GNU General Public License
-;; along with Guile-PG; if not, write to the Free Software Foundation,
-;; Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+;; along with Guile-PG.  If not, see <http://www.gnu.org/licenses/>.
 
 (or (not *all-tests*)
     (begin (display "Bad env.")
            (newline)
            (exit #f)))
 
-(define (-fs dest)
-  (lambda (s . args)
-    (apply simple-format dest s args)))
-
-(define fs (-fs #f))
-(define fso (-fs #t))
+(cond-expand (guile-2 (read-disable 'square-brackets))
+             (else #f))
 
 (use-modules (database postgres)
              (database postgres-qcons)
@@ -38,9 +32,7 @@
              (srfi srfi-13)
              (srfi srfi-14)
              (ice-9 regex)
-             (ice-9 common-list))
-
-(fresh!)
+             (srfi srfi-1))
 
 (define db-name (or (getenv "PGDATABASE")
                     (error "don't know what database to use")))
@@ -48,7 +40,7 @@
 (define (mgr . args)
   (let ((rv (apply pgtable-manager args)))
     (cond ((procedure? rv)
-           (and (getenv "DEBUG")
+           (and (env-1? "DEBUG")
                 ((rv #:trace-exec) *log-file*))
            (let ((scs 'standard_conforming_strings)
                  (conn ((rv #:k) #:connection)))
@@ -68,7 +60,7 @@
                             "UTF8")))
              (cond ((pg-parameter-status conn scs)
                     => (lambda (v)
-                         (simple-format #t "INFO: ~S => ~S~%" scs v)
+                         (fso "INFO: ~S => ~S~%" scs v)
                          (fluid-set! sql-quote-auto-E? #t)))))))
     rv))
 
@@ -85,7 +77,7 @@
   (let ((sym (pg-result-status raw)))
     (case sym
       ((PGRES_COMMAND_OK) #t)
-      (else (and (equal? "1" (getenv "DEBUG"))
+      (else (and (env-1? "DEBUG")
                  (fso "~A: ~A~%" sym (pg-result-error-message raw)))
             #f))))
 
@@ -123,8 +115,8 @@
      (pass-if "result status"
        (let ((s (pg-result-status tupres)))
          (cond ((eq? 'PGRES_TUPLES_OK s))
-               (else (format #t "EWHY: result status: ~A\n      ~A\n"
-                             s (pg-result-error-message tupres))
+               (else (fso "EWHY: result status: ~A~%      ~A~%"
+                          s (pg-result-error-message tupres))
                      #f))))
      ,@body))
 
@@ -134,8 +126,15 @@
 ;; expect #t
 ;;
 (define (test:query-oid/type-name)
-  (->bool (oid-type-name-cache
-           (pg-connectdb (fs "dbname=~A" db-name)))))
+  (let* ((conn (pg-connectdb (fs "dbname=~A" db-name)))
+         (cache (oid-type-name-cache conn)))
+    (pg-finish conn)
+    (and (pair? cache)
+         (and-map (lambda (ent)
+                    ;; (fso "ent: ~S~%" ent)
+                    (and (integer? (car ent))
+                         (string? (cdr ent))))
+                  cache))))
 
 ;; Test pgtable-manager
 ;; expect #t
@@ -151,13 +150,13 @@
             ;;                        application that wrapped rsync)
             '((time            timestamp)
               (error_condition text)
-              (files           text[])
+              (files           *text)
               (wrote           int4)
-              (read            text[][])
+              (read            **text)
               (rate            float4)
               (total           int4)
               (speedup         float4)
-              (etc             int4[])))))
+              (etc             *int4)))))
   (procedure? m))
 
 ;; Test pgtable-manager procs
@@ -206,20 +205,25 @@
                           (display ")")))
                       (map cddr PESKY)))
 
+(define excruciatingly-correct-string-filter
+  (cond-expand (guile-2 string-filter)
+               (else (lambda (pred s . rest)
+                       (apply string-filter s pred rest)))))
+
 (define POISON-COUNTS (let ((adj (+ (string-length
                                      (object->string
                                       (make-list (length PESKY)
                                                  "")))
                                     (string-length
-                                     (string-filter
-                                      POISON
+                                     (excruciatingly-correct-string-filter
                                       (char-set #\\ #\")
+                                      POISON
                                       (string-index POISON #\)))))))
                         (define (sum sel)
                           (+ adj (* 2 (apply + (map sel PESKY)))))
                         (fs "~A ~A" (sum car) (sum cadr))))
 
-(and (getenv "DEBUG")
+(and (env-1? "DEBUG")
      (fso "POISON: ~A |~A|~%" POISON-COUNTS POISON))
 
 ;; Test pgtable-manager insert 2
@@ -260,7 +264,7 @@
              (pass-if "size" (check-dim 1 10))
              (pass-if "ec" (let ((counts (tref 0 9))
                                  (string (tref 0 1)))
-                             (and (getenv "DEBUG")
+                             (and (env-1? "DEBUG")
                                   (fso "ec: ~A |~A|~%" counts string))
                              (and (string=? POISON-COUNTS counts)
                                   (string=? POISON string))))
@@ -324,7 +328,7 @@
   (cons 'misc-error "malformed def"))
 
 (define-macro (expect-bad-defs defs)
-  `(pass-if-exception (format #f "~S" ,defs)
+  `(pass-if-exception (fs "~S" ,defs)
        pgtable:exception:malformed-defs
      (pgtable-manager db-name "dontcare" ,defs)))
 
@@ -490,7 +494,6 @@
   (test-m2)
   (test-m3)
   (cleanup!)
-  (drop!)
   (test-report))
 
 (exit (main))
